/*
 * The MIT License (MIT)
 *
 * Copyright 2015-2021 Valentyn Kolesnikov
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package com.github.underscore;

import static org.junit.Assert.assertEquals;

import java.util.*;
import org.junit.Test;

/**
 * Underscore library unit test.
 *
 * @author Valentyn Kolesnikov
 */
public class ChainingTest {

    /*
    var stooges = [{name: 'curly', age: 25}, {name: 'moe', age: 21}, {name: 'larry', age: 23}];
    var youngest = _.chain(stooges)
      .sortBy(function(stooge){ return stooge.age; })
      .map(function(stooge){ return stooge.name + ' is ' + stooge.age; })
      .first()
      .value();
    => "moe is 21"
    */
    @Test
    @SuppressWarnings("unchecked")
    public void chain() {
        final List<Map<String, Object>> stooges =
                new ArrayList<Map<String, Object>>() {
                    {
                        add(
                                new LinkedHashMap<String, Object>() {
                                    {
                                        put("name", "curly");
                                        put("age", 25);
                                    }
                                });
                        add(
                                new LinkedHashMap<String, Object>() {
                                    {
                                        put("name", "moe");
                                        put("age", 21);
                                    }
                                });
                        add(
                                new LinkedHashMap<String, Object>() {
                                    {
                                        put("name", "larry");
                                        put("age", 23);
                                    }
                                });
                    }
                };
        final String youngest =
                U.chain(stooges)
                        .sortBy(
                                (item) -> {
                                    return (Integer) ((Map<String, Object>) item).get("age");
                                })
                        .map(
                                (item) -> {
                                    return item.get("name") + " is " + item.get("age");
                                })
                        .first()
                        .item();
        assertEquals("moe is 21", youngest);
    }
}
